#!/bin/bash

# Test de la connectivite des ordinateurs Linux
INTERFACE="eth0"

################## FIN DE LA CONFIGURATION ###################################

##### NE PAS MODIFIER CI- DESSOUS #####

PATH="/sbin:/bin:/usr/bin"

if [ "$1" != "" ]; then 
  INTERFACE=$1
fi

function afficheERREUR ()
{
  echo -n "["
  echo -en "\E[31;48m"
  echo -n "ERREUR"
  tput sgr0
  echo "]"
}
function afficheALERTE ()
{
  echo -n "["
  echo -en "\E[33;48m"
  echo -n "ALERTE"
  tput sgr0
  echo "]"
}
function afficheOK ()
{
  echo -n "["
  echo -en "\E[32;48m"
  echo -n "  OK  "
  tput sgr0
  echo "]"
}

echo "Vérification des informations locales :"
echo "---------------------------------------"
echo "Toutes ces informations sont celles configurées dans votre machine. "
echo "Si une erreur est signalée, vous devez configurer votre Linux "
echo "correctement."
echo ""

# Verification de l'activation de l'interface eth0
echo -n "Vérification de l'activation de $INTERFACE :                                "
if [ "`netstat -i | grep "$INTERFACE"`" == "" ]; then
  afficheERREUR
  echo "   L'interface $INTERFACE n'est pas activée : le réseau ne peut pas"
  echo "   fonctionner. Activez l'interface ou, si ce n'est pas votre interface"
  echo "   de connexion, relancez ce programme en mettant l'interface en "
  echo "   argument :"
  echo "   Ex $0 ppp0"
  exit
fi
afficheOK

# Recuperation de l'adresse IP de la machine
echo -n "Récupération de l'adresse IP de l'interface $INTERFACE :                    "
LIGNEIP=`ifconfig $INTERFACE | grep "inet addr"`
if [ "$LIGNEIP" == "" ]; then
  afficheERREUR
  echo "   L'interface réseau $INTERFACE n'est pas configurée."
  echo "   Le problème peut être du à un câble réseau mal branché (essayez de "
  echo "   le déconnecter/reconnecter), un routeur qui n'a pas distribué "
  echo "   d'adresse IP (redémarrer votre Freebox). En cas d'adressage "
  echo "   statique, il faut configurer votre Linux (dans Ubuntu, "
  echo "   Système/Administration/Réseau)"
  exit
fi
IPADDR=`echo $LIGNEIP | awk '{print $2}'|awk -F ":" '{print $2}'`
afficheOK

# Verification de la classe d'adresse IP (classe non routable)
# selon l'IANA
echo -n "Vérification si l'adresse est non routable :                          "
if [ "`echo $IPADDR | cut -d '.' -f 1-2`" == "192.168" ] ||
   [ "`echo $IPADDR | cut -d '.' -f 1`"   == "10" ] ||
   ( [ "`echo $IPADDR | cut -d '.' -f 1`"   == "172" ] && 
       [ `echo $IPADDR | cut -d '.' -f 2` >= 16 ] && 
       [ `echo $IPADDR | cut -d '.' -f 2` <= 31 ]
   ) ; then
  afficheALERTE
  echo "   Votre adresse est non routable : ce n'est pas grave si votre Freebox"
  echo "   est en mode routeur. Si elle est en bridge, vous n'aurez pas de "
  echo "   connexion Internet."
  echo "   SOLUTION EN MODE ROUTEUR : Aucune, c'est une adresse normale"
  echo "   SOLUTION EN MODE BRIDGE :  Mettre en DHCP ou configurez en statique"
  echo "     votre Linux (sous Ubuntu, Système/Administration/Réseau)"
else
  afficheOK
fi

# Verification de la route par defaut
echo -n "Vérification de l'existence de la route par défaut :                  "
ROUTE=`netstat -rn | grep "^0.0.0.0" | awk '{print $2}'`
if [ "$ROUTE" == "" ]; then
  afficheERREUR
  echo "   Vous n'avez pas de route par défaut : votre connexion Internet ne "
  echo "   peut fonctionner"
  echo "   SOLUTION : Réactivez la route par défaut en désactivant l'interface"
  echo "     $INTERFACE puis en la réactivant."
  echo "     Sur Ubuntu, Système/Administration/Réseau choisir l'interface"
  echo "     $INTERFACE puis cliquez sur désactiver, puis réactiver"
  exit
fi
afficheOK

# Verification de la coherence de la route par defaut avec l'adresse IP et le
# masque
# je fais simple : si les 3 premiers nombres de l'adresse IP correspondent
# avec les 3 premiers nombres de la passerelle : OK
# ca ne fonctionne pas pour les personnes en classe A ou B mais est-ce que
# ca existe ?
# Devra etre ameliore ulterieurement.
echo -n "Vérification de la cohérence de la route par défaut :                 "
if [ "`echo $ROUTE | cut -d '.' -f 1-2`" != \
     "`echo $IPADDR | cut -d '.' -f 1-2`" ]; then
  afficheERREUR
  echo "   Votre route par défaut ne correspond pas à votre adresse IP."
  echo "   votre connexion Internet ne peut fonctionner"
  echo "   SOLUTION : Vérifiez votre configuration."
  echo "     Sur Ubuntu, Système/Administration/Réseau, choisissez l'interface"
  echo "     $INTERFACE, Propriétés, et vérifiez la passerelle"
  exit
fi
afficheOK

# Verification de l'existence d'un (ou plusieurs) serveurs DNS
echo -n "Vérification de l'existence d'un (ou plusieurs) serveurs DNS :        "
DNS1=`grep -m 1 "^nameserver" /etc/resolv.conf`
if [ "$DNS1" == "" ]; then
  afficheERREUR
  echo "   Votre configuration ne comporte pas de serveur de nom."
  echo "   Vous ne pourrez pas accéder aux sites en tapant leur nom"
  echo "   seulement leur adresse IP"
  echo "   SOLUTION : Insérez un serveur DNS dans votre configuration."
  echo "     Sur Ubuntu, Système/Administration/Réseau, onglet DNS"
  echo "     Insérez les DNS : "
  echo "     Pour Free : 212.27.54.252 et 212.27.53.252 "
else
  afficheOK
fi

# Verification de la presence d'un firewall
echo -n "Vérification de la présence d'un firewall (ou pare-feu) :             "
if [ "`lsmod | grep ip_tables`" == "" ]; then
  afficheALERTE
  echo "   Le système de pare-feu n'est pas activé sur votre ordinateur."
  echo "   Il se peut que celui-ci coure un risque."
  echo "   SOLUTION : Installez un pare-feu (comme Firestarter)"
  echo "     Sur Ubuntu, Système/Administration/Gestionnaire de package "
  echo "     Synaptics, chercher Firestarter et l'installer."
  FW=0
else
  afficheALERTE
  echo "   Le système de pare-feu est activé sur votre ordinateur."
  echo "   Les tests de connectivité peuvent être erronés si le pare-feu"
  echo "   rejette les tests."
  FW=1
fi 
# FIN DES VERIFICATIONS LOCALES

# Verifications Reseau
echo ""
echo "Vérification du fonctionnement du réseau (tests de connectivité) :"
echo "------------------------------------------------------------------"

# Test du ping vers l'adresse locale
echo -n "Test ping de l'adresse locale :                                       "
ping -n -c 1 -W 1 -w 2 $IPADDR > /dev/null
if [ $? -ne 0 ]; then
  afficheERREUR
  echo "   L'interface locale ne répond pas au ping"
  echo "   SOLUTION : Redémarrez l'interface $INTERFACE"
  echo "     Sur Ubuntu, Système/Administration/Réseau choisir l'interface"
  echo "     $INTERFACE puis cliquez sur désactiver, puis réactiver"
  exit
fi
afficheOK

# Test du ping vers la passerelle
echo -n "Test ping de la passerelle par défaut :                               "
ping -n -c 1 -W 1 -w 2 $ROUTE > /dev/null
if [ $? -ne 0 ]; then
  afficheERREUR
  echo "   La passerelle par défaut ne répond pas au ping"
  echo "   SOLUTION : Vérifiez câbles, switchs (si il y en a), que la "
  echo "     Freebox soit active. Redémarrez la Freebox si nécessaire."
  echo "     Essayez de déconnecter/reconnecter le câble réseau."
  echo "     Vérifiez que la Freebox affiche l'heure"
  exit
fi
afficheOK

# Test du DNS
echo -n "Test de la résolution de noms :                                       "
host -W 1 www.google.fr > /dev/null
if [ $? -ne 0 ]; then
  afficheERREUR
  echo "   La résolution de nom pour www.google.fr ne fonctionne pas."
  echo "   Vous ne pouvez pas avoir accès aux sites en tapant leur nom."
  echo "   SOLUTION : Si le DNS a déjà fonctionné, c'est le serveur de Free"
  echo "     qui est en panne. Attendez que ce soit réparé."
  echo "     Si votre DNS n'a jamais fonctionné, vérifiez que vous avez bien"
  echo "     inséré l'adresse du DNS dans votre configuration. Sur Ubuntu,"
  echo "     Système/Administration/Réseau, onglet DNS"
  echo "     Insérez les DNS : "
  echo "     Pour Free : 212.27.54.252 et 212.27.53.252 "
  exit
fi
afficheOK


# Ici test termine
echo ""
echo "Le test est terminé et le réseau fonctionne correctement."
echo "Si vous ne pouvez pas joindre un site particulier, essayez de vous "
echo "reconnecter un peu plus tard."
